from django.shortcuts import render, redirect, render_to_response
from django.shortcuts import get_object_or_404, render
from django.http import HttpResponse, JsonResponse
from django.template import RequestContext, loader, Context
from django.core.urlresolvers import reverse_lazy
from django.views.generic import ListView, CreateView, UpdateView, DeleteView, View
from apps.prueba.forms import PruebaForm
from apps.prueba.models import Prueba, Resultado
from django.db.models import Avg, F, FloatField, Sum, Max
from django.core import serializers




import math
import matplotlib.pyplot as plt
import csv, operator, os, psycopg2, psycopg2.extras
#Graficas
#% matplotlib inline
#% config InlineBackend.figure_format = svg

class PruebaList(ListView):
	model = Prueba
	template_name = 'prueba/prueba_list.html'
	
class PruebaCreate(CreateView):
	model = Prueba
	form_class = PruebaForm
	template_name = 'prueba/prueba_form.html'
	success_url = reverse_lazy('prueba:listar')
	
class PruebaUpdate(UpdateView):
	model = Prueba
	form_class = PruebaForm
	template_name = 'prueba/prueba_form.html'
	success_url = reverse_lazy('prueba:listar')
	
class PruebaDelete(DeleteView):
	model = Prueba
	template_name = 'prueba/prueba_delete.html'
	success_url = reverse_lazy('prueba:listar')
	
class ResultadoList(ListView):
	model = Resultado
	template_name = 'prueba/prueba_list.html'
	
class ResultadoDelete(DeleteView):
	model = Resultado
	template_name = 'prueba/resultado_delete.html'
	success_url = reverse_lazy('prueba:listar')

def convertirhora(ent):
	h = str(int(ent) // 3600)
	minu = str(int(ent) % 3600 // 60)
	r = str(int(ent) % 60)
	hora = h + ':' + minu + ':' + r
	return hora

def validar(request):
	lista = request.POST.getlist('checkbox')
	d = request.POST['distancia_objetivo']
	ld = len(d)
	#Longitud de los datos
	lon = len (lista)
	suma = Prueba.objects.filter(id__in=lista).aggregate(Sum('distancia')).values()
	# Recorrer el diccionario de la sumatoria de la distancia para volverlo número
	for sl in suma:
		sdiss = sl
	prom =sdiss/lon
	if ((lon < 2)or(ld == 0)):
		msj="Debe seleccionar más de dos distancias diferentes y/o ingresar la distancia objetivo"
		if ((prom == 50)or(ld == 0)):
			msj="Debe seleccionar distancias diferentes de 50 y/o la distancia objetivo"
		if ((prom == 100)or(ld == 0)):
			msj="Debe seleccionar distancias diferentes de 100 y/o la distancia objetivo"
		if ((prom == 200)or(ld == 0)):
			msj="Debe seleccionar distancias diferentes de 200 y/o la distancia objetivo"
		if ((prom == 400)or(ld == 0)):
			msj="Debe seleccionar distancias diferentes de 400 y/o la distancia objetivo"
		if ((prom == 800)or(ld == 0)):
			msj="Debe seleccionar distancias diferentes de 800 y/o la distancia objetivo"
		if ((prom == 1500)or(ld == 0)):
			msj="Debe seleccionar distancias diferentes de 1500 y/o la distancia objetivo"
		return HttpResponse(msj)
	else: 
		return ritmo(request)
	
	

def ritmo(request):
	lista = request.POST.getlist('checkbox')
	d = request.POST['distancia_objetivo']
	disobj = int(d)
	# Calculo del ritmo
	a = Prueba.objects.filter(id__in=lista).annotate(a=(F('tiempo')/F('distancia')*100))
	prueba = Prueba.objects.all().order_by('id') #queryset
	#Longitud de los datos
	lon = len (lista)
	#-------------------------------------------------------------------------------#
	#-------------------------------------------------------------------------------#
	# Sumatoria de la distancia
	sumd = Prueba.objects.filter(id__in=lista).aggregate(Sum('distancia')).values()
	# Recorrer el diccionario de la sumatoria de la distancia para volverlo número
	for n in sumd:
		sdis = n
	#-------------------------------------------------------------------------------#
	# Sumatoria del tiempo
	sumt = Prueba.objects.filter(id__in=lista).aggregate(Sum('tiempo')).values()
	# Recorrer el diccionario de la sumatoria del tiempo para volverlo número
	for m in sumt:
		stmp = m
	#-------------------------------------------------------------------------------#	
	# Media de la distancia
	mdis = Prueba.objects.filter(id__in=lista).aggregate(dis=(Avg('distancia'))).values()
	# Recorrer el diccionario de la media de la distancia para volverlo número
	for o in mdis:
		md = o
	#-------------------------------------------------------------------------------#
	# Media del tiempo
	mtmp = Prueba.objects.filter(id__in=lista).aggregate(tiempo=(Avg('tiempo'))).values()
	for r in mtmp:
		mt = r
	#-------------------------------------------------------------------------------#
	# Sumatoria de distancia por el tiempo
	distmp = Prueba.objects.filter(id__in=lista).aggregate(a=(Sum(F('distancia')*F('tiempo')))).values()
	for p in distmp:
		sdt = p
	#-------------------------------------------------------------------------------#
	# Sumatoria de la distancia al cuadrado
	dist2 = Prueba.objects.filter(id__in=lista).aggregate(a=(Sum(F('distancia')*F('distancia')))).values()
	#dist2 = Prueba.objects.filter(id__in=lista).extra(select={"distancia": "(distancia * distancia)"}, where=["distancia > 10"]).aggregate(Sum('distancia')).values()
	for q in dist2:
		dis2 = q
	#-------------------------------------------------------------------------------#
	# Sumatoria de la tiempo al cuadrado
	tim2 = Prueba.objects.filter(id__in=lista).aggregate(a=(Sum(F('tiempo')*F('tiempo')))).values()
	for s in tim2:
		tmp2 = s
	#-------------------------------------------------------------------------------#
	# Calculo de m y b	
	m = (sdis*stmp - lon*sdt)/(stmp**2 - lon*tmp2)
	b = md - m*mt
	# Calculo del tiempo estimado y el ritmo estimado toca cambiar ese 1000 por lo que entra en el formulario
	tiemE = (disobj - b ) / m
	tiemE2 = convertirhora (tiemE)
	ritmE = tiemE / disobj * 100
	ritmE2 = convertirhora (ritmE)
	#-------------------------------------------------------------------------------#
	# Calculo de sigma x
	sigmax = math.sqrt(tmp2 / lon - mt**2)
	sigmay = math.sqrt(dis2 / lon - md**2)
	sigmaxy = (sdt/lon) - (mt*md)
	coeficiente = (sigmaxy / (sigmax*sigmay))**2
	# Convertir m a horas, minutos y segudos
	#Calcular la velocidad critica
	css = 100/m
	css2 = convertirhora(css)
	# Calcular el Factor Fatiga
	# Filtrar datos y aplica logaritmo natural a cada valor en la consulta
	#d = Prueba.objects.filter(id__in=lista).extra(select={"distancia": "sum(ln (distancia))"}, where=["distancia > 10"]).values('distancia')
	#t = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "sum(ln (tiempo))"}, where=["tiempo > 0"]).values('tiempo')
    # Calcular m con ln 
	#mln = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "sum(ln (tiempo))*sum(ln (tiempo))"}, where=["tiempo > 0"]).values('tiempo')
	#mdt = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "sum(ln (distancia))*sum(ln (tiempo))"}, where=["tiempo > 0"]).values('tiempo')
	#mtln = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "4 * (sum((ln (tiempo))*(ln (tiempo))))"}, where=["tiempo > 0"]).values('tiempo')	
	#mdt = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "(sum(ln (distancia))*sum(ln (tiempo)))-(4 * (sum((ln (tiempo))*(ln (distancia)))))"}, where=["tiempo > 0"]).values('tiempo')
	#mln1 = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "(sum(ln (tiempo))*sum(ln (tiempo)))-(4 * (sum((ln (tiempo))*(ln (tiempo)))))"}, where=["tiempo > 0"]).values('tiempo')
		
	mln2 = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "((sum(ln (distancia))*sum(ln (tiempo)))-(count(*) * (sum((ln (tiempo))*(ln (distancia))))))/((sum(ln (distancia))*sum(ln (distancia)))-(count(*) * (sum((ln (distancia))*(ln (distancia))))))"}, where=["tiempo > 0"]).values_list('tiempo', flat=True)
	bln2 = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "(sum((ln (tiempo)))/count(*))-(((sum(ln (distancia))*sum(ln (tiempo)))-(count(*) * (sum((ln (tiempo))*(ln (distancia))))))/((sum(ln (distancia))*sum(ln (distancia)))-(count(*) * (sum((ln (distancia))*(ln (distancia)))))))*(sum((ln (distancia)))/count(*))"}, where=["tiempo > 0"]).values('tiempo').values_list('tiempo', flat=True)
	# Calculo de m, donde m es val
	valor = list (mln2)
	for num in valor:
		val = num
	val3 = float(val)
	val2 = convertirhora(val)
	# Calculo de b representada por bln
	valb = list (bln2)
	for numb in valb:
		valb = numb
	bln = math.exp(valb)
	# Tiempo estimado con FF
	tieFF = bln * disobj**val 
	tieFF2 = convertirhora(tieFF)
		
	# Calcular el coeficiente para el factor fátiga
	tln2 = Prueba.objects.filter(id__in=lista).extra(select={"tiempo": "sum((ln (tiempo))*(ln (tiempo)))"}, where=["tiempo > 0"]).values('tiempo').values_list('tiempo', flat=True)
	dln2 = Prueba.objects.filter(id__in=lista).extra(select={"distancia": "sum((ln (distancia))*(ln (distancia)))"}, where=["tiempo > 0"]).values('distancia').values_list('distancia', flat=True)
	mdln2 = Prueba.objects.filter(id__in=lista).extra(select={"mediadis": "AVG(ln (distancia))"}, where=["tiempo > 0"]).values('mediadis').values_list('mediadis', flat=True)
	mtln2 = Prueba.objects.filter(id__in=lista).extra(select={"mediatm": "AVG(ln (tiempo))"}, where=["tiempo > 0"]).values('mediatm').values_list('mediatm', flat=True)
	mtdln = Prueba.objects.filter(id__in=lista).extra(select={"tiedis": "(sum((ln (tiempo))*(ln (distancia))))"}, where=["tiempo > 0"]).values('tiedis').values_list('tiedis', flat=True)
	#Sacar cada valor de la consulta
	# tiempo^2
	ltmp = list(tln2)
	for ntc in ltmp:
		valcoef = ntc
	# distancia ^2
	ldis = list (dln2)
	for ndc in ldis:
		discoef = ndc
	# Media distancia
	lmdis = list (mdln2)
	for mdc in lmdis:
		mlnd = mdc
	# Medis del tiempo
	lmtie = list (mtln2)
	for mtc in lmtie:
		mlnt = mtc
	# Media del tiempo por la distancia
	lmtxd = list (mtdln)
	for mtxd in mtdln:
		mlndxt = mtxd

	siglanx = math.sqrt(valcoef / lon - mlnt**2)
	siglany = math.sqrt(discoef / lon - mlnd**2)
	siglnxy = (mlndxt/lon ) - (mlnt*mlnd)

	coef = (siglnxy / (siglanx*siglany))**2

	# Cálculo del ritmo con FF
	ritFF = tieFF / disobj * 100
	ritFF2 = convertirhora(ritFF)


	# Calcular la tabla de ritmos estimados por nivel Phil skiba
	res = css * 1.07
	res2 = convertirhora(res)
	tem = css * 1.03
	tem2 = convertirhora(tem)
	umbr = css * 1.01
	umbr2 = convertirhora(umbr)
	vo = css * 0.98
	vo2 = convertirhora(vo)
	cap = css * 0.92
	cap2 = convertirhora(cap)
	
	# Calcular la tabla de ritmos estimados por nivel Bill sweetenham
	blne = math.exp(valb) # b de ln
	test = blne * 200**val
	testh = convertirhora(test)
	test2 = test / 2
	test2h = convertirhora(test2)
	z1a1 = test2 + 20
	z1a12 = convertirhora(z1a1)
	z1a2 = test2 + 15
	z1a22 = convertirhora(z1a2)
	z1a3 = test2 + 10
	z1a32 = convertirhora(z1a3)
	z2= test2 + 7
	z22 = convertirhora(z2)
	z3 = test2 + 4
	z32 = convertirhora(z3)
	
	#convirtiendo valores a datetime
	hour = "0:01:41"
	ho,mi,se = hour.split(":")
	hor =  (float(ho) / 24.0 + float(mi) / (24.0*60.0) + float(se) / (24.0*60.0*60.0))*100
	


	
	
	#return HttpResponse(hor)
	#return render(request, 'prueba/resultado_list.html', {'lista': mln2})

	return render_to_response ('prueba/resultado_list.html',
	                          {'resultados':prueba, 'resultados': a, 'css': css2, 'css2': css, 'coef': coeficiente, 'ff': val, 'ff2': val3, 'te' : tiemE2, 'te2' : tiemE, \
	                          're' : ritmE2, 're2' : ritmE, 'dis': disobj, 'cefff': coef, 'ritFF': ritFF2, 'ritFF2': ritFF, 'tieFF': tieFF2, 'tieFF2': tieFF, 'res' :res2, \
	                          'res2' :res, 'tem' :tem2, 'tem2' :tem,\
	                          'umbr' :umbr2, 'umbr2' :umbr, 'vo' :vo2, 'vo2' :vo,'cap' :cap2, 'cap2' :cap, 'valb': testh, 'test2': test2h, 'test2h': test,\
	                          'z1a1': z1a12, 'z1a12': z1a1, 'z1a2': z1a22, 'z1a22': z1a2, 'z1a3': z1a32, 'z1a32': z1a3, 'z2': z22, 'z22': z2, 'z3': z32, 'z32': z3,},
	                        context_instance=RequestContext (request))
					
	#y=np.log(100)
	#x = Prueba.objects.filter(id__in=lista).values('distancia')
	#y = np.array(x)
	#for dia, codigos in y.items():
	
	#contexto = {'resultados':di} #contexto
	#return render(request, 'prueba/resultado_list.html', contexto)
	
def carga(request):
	return render(request, 'prueba/prueba_carga.html')

def guardar(request):
	bd = psycopg2.connect(database='rendimiento',user='postgres',password='admin', host='localhost')
	cursor = bd.cursor()
	#cursor.execute("SELECT * FROM prueba_prueba")
	#rows=cur.fetchall()
	nombre = request.POST.get('csv_file')
	directorio = os.getcwd()
	ruta = str(directorio) + '/archivo/'
	filePath = os.path.relpath(ruta+nombre)
	csvarchivo = open(filePath)  # Abrir archivo csv
	entrada = csv.reader(csvarchivo)  # Leer todos los registros
	for line in entrada:
		id = line[0]
		distancia = line[1]
		tiempo =  line [2]
		query = "INSERT INTO prueba_prueba (id, distancia, tiempo) VALUES (%s, %s, %s);"
		data = (id, distancia, tiempo)
		cursor.execute(query, data)
	cursor.execute('SELECT * FROM prueba_prueba')
	for row in cursor:
		rows = row
	bd.commit()
	bd.close()
	return render(request, 'prueba/prueba_ok.html')


def ayuda(request):
	return render(request, 'prueba/prueba_ayuda.html')
	


	#return render(request, 'prueba/prueba_list.html')

#def index_prueba(request):
#	return render(request, 'prueba/index_prueba.html')
import json
class HomeView(View):
	def get(self, request, *args, **kwargs):
		return render(request, 'prueba/index_prueba.html', {})	



	
#def get_data (request):
	#obtener todas las zonas
#	zonas = Prueba.objects.all()

	#construir una lista de diccionarios que representen a las zonas
#	lista = [{'pk': zona.pk, 'zona': zona.distancia, 'tzona': zona.tiempo} for zona in zonas]

	#convertir eso en un json

#	serializado = json.dumps(lista)
#	return HttpResponse(serializado, content_type='application/json')

	
#def myModel_asJson(request):
 #   object_list = Prueba.objects.all() #or any kind of queryset
  #  json = serializers.serialize('json', object_list)
  

def get_data(request):
    tasks = Prueba.objects.all()
    data = serializers.serialize("json", tasks)
    return HttpResponse(data, content_type='application/json')

def index_prueba(request):
    suppliers_all = Prueba.objects.all().values('distancia', 'tiempo')
    suppliers_all = json.dumps(list(suppliers_all))
    context = {'suppliers_all': suppliers_all,
               }
    return render(request, 'prueba/index_prueba.html', context)

	
